<?php
/**
 * @copyright	Copyright (C) 2009-2016 ACYBA SAS - All rights reserved..
 * @license		GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */
defined('_JEXEC') or die('Restricted access');

if(!defined('DS')) define('DS',DIRECTORY_SEPARATOR);

class plgSystemVmacymailing extends JPlugin
{

	var $triggered = false;

	public function __construct(&$subject, $config){
		parent::__construct($subject, $config);
		$file = rtrim(JPATH_ROOT,DS).DS.'administrator'.DS.'components'.DS.'com_virtuemart'.DS.'version.php';
		if(!file_exists($file)) return false;
		include_once($file);
		$vmversion = new vmVersion();
		$this->version = empty($vmversion->RELEASE) ? vmVersion::$RELEASE : $vmversion->RELEASE;
	}

	public function plgVmConfirmedOrder($all,$order){
		if($this->triggered) return;
		$this->triggered = true;
		//This is for VM2
		//The user just confirmed his order on VM, we will add him in Acy.
		if(empty($order['details']['BT'])) return;

		$this->_subscribeUser($order['details']['BT']);
	}

	//Same plgVmConfirmedOrder trigger but for OnePageCheckout component.
	public function plgVmConfirmedOrderOPC($type,$all,$order){
		return $this->plgVmConfirmedOrder($all,$order);
	}

	public function plgVmAfterUserStore($userData){
		if(empty($userData)) return;
		$this->_subscribeUser($userData);
	}

	public function plgVmOnUserStore($data){
		$this->_subscribeUser($data);
		return $data;
	}

	private function _subscribeUser($userInfos){
		if(!is_array($userInfos)) $userInfos = get_object_vars($userInfos);

		//Check the parameter which determines if the user should be subscribed or not...
		$subscribeField = $this->params->get('checkfield','user_email');
		if(version_compare($this->version,'2.0.0','>=')){
			if($subscribeField == 'virtuemart_userinfo_id') $subscribeField = 'virtuemart_order_userinfo_id';
			if($subscribeField == 'name') $subscribeField = 'last_name';
		}

		$emailValue = @$userInfos['email'];
		if($this->params->get('multiLists', 'no') != 'no'){
			if(!is_string($userInfos[$subscribeField])){
				// guest mode is not working in VM3
				if(version_compare($this->version,'3.0.0','>=') && $userInfos[$subscribeField] === null) return;
				$userInfos[$subscribeField] = implode('|', $userInfos[$subscribeField]);
			}
			$listsFromVM = @$userInfos[$subscribeField];
			$listsFromVM = str_replace('|',',',$listsFromVM);
			// The field doesn't contain list
			if(empty($listsFromVM)) return;
		}else{
			$subscribeValue = (($subscribeField == 'user_email' || $subscribeField == 'email') ? $emailValue : @$userInfos[$subscribeField]);
			//The field that should be checked is not checked so we don't add the user and can return immediately
			if(empty($subscribeValue) OR empty($emailValue) OR in_array(strtolower($subscribeValue),array('no','none','nein','non'))) return;
		}

		//We include the acymailing helper function as we will need it!
		if(!include_once(rtrim(JPATH_ADMINISTRATOR,DIRECTORY_SEPARATOR).DIRECTORY_SEPARATOR.'components'.DIRECTORY_SEPARATOR.'com_acymailing'.DIRECTORY_SEPARATOR.'helpers'.DIRECTORY_SEPARATOR.'helper.php')) return;

		//Now we can subscribe the user depending on the params...
		//Step 1 : insert the user, that will do an update in case of the user is already inserted.
		$user = new stdClass();
		$user->email = trim(strip_tags($emailValue));

		//Avoid any problem...
		$userHelper = acymailing_get('helper.user');
		if(!$userHelper->validEmail($user->email)) return;

		$user->name = '';
		if(!empty($userInfos['first_name'])) $user->name .= $userInfos['first_name'].' ';
		if(!empty($userInfos['middle_name'])) $user->name .= $userInfos['middle_name'].' ';
		if(!empty($userInfos['last_name'])) $user->name .= $userInfos['last_name'];
		$user->name = trim(strip_tags($user->name));

		if($this->params->get('subconf','default') == 'yes') $user->confirmed = 1;

		$userClass = acymailing_get('class.subscriber');
		$userClass->checkVisitor = false;
		$userClass->sendConf = false;
		if($this->params->get('sendconf','no') == 'yes') $userClass->sendConf = true;

		$my = JFactory::getUser();
		if(!empty($my->id)){
			$user->userid = $my->id;
			$user->subid = $userClass->subid($my->id);
		}

		//We may update the user with a new e-mail address... lets make sure we can do that and we won't have a conflict with an existing address
		//Thats only for an update and only if that e-mail address exists for another user...
		if(!empty($user->subid)){
			$currentSubid = $userClass->subid($user->email);
			if(!empty($currentSubid) && $user->subid != $currentSubid){
				//The user already exists and has a different subid... that will block the update.
				//We will delete that user.
				$userClass->delete($currentSubid);
			}
		}

		//Here we updated the user or added a new one properly in all cases
		$subid = $userClass->save($user);

		if(empty($subid)) return;


		if(!empty($listsFromVM)){
			$listsToSubscribe = $listsFromVM;
		} else{
			$listsToSubscribe = $this->params->get('lists','all');
		}
		$config = acymailing_config();

		$listsClass = acymailing_get('class.list');
		$allLists = $listsClass->getLists('listid');
		if(acymailing_level(1)){
			$allLists = $listsClass->onlyCurrentLanguage($allLists);
		}

		$listsArray = array();
		if(strpos($listsToSubscribe,',') OR is_numeric($listsToSubscribe)){
			$listsArrayParam = explode(',',$listsToSubscribe);
			foreach($allLists as $oneList){
				if($oneList->published && in_array($oneList->listid,$listsArrayParam)){$listsArray[] = $oneList->listid;}
				if(!empty($listsFromVM) && $oneList->published && !in_array($oneList->listid,$listsArrayParam)){$remListsArray[] = $oneList->listid;}
			}
		}elseif(strtolower($listsToSubscribe) == 'all'){
			foreach($allLists as $oneList){
				if($oneList->published){$listsArray[] = $oneList->listid;}
			}
		}

		if(empty($listsArray)) return;

		//Get the saved subscriber to make sure it's updated and the good one
		$inserteduser = $userClass->get($subid);
		$currentSubscription = $userClass->getSubscriptionStatus($subid);

		$statusAdd = (empty($inserteduser->confirmed) AND $config->get('require_confirmation',false)) ? 2 : 1;

		$addlists = array();
		foreach($listsArray as $idOneList){
			//The user is not already subscribed to this list... so we add it
			if(!isset($currentSubscription[$idOneList])){
				$addlists[$statusAdd][] = $idOneList;
			}
		}

		if(!empty($remListsArray)){
			foreach($remListsArray as $idOneList){
				if(isset($currentSubscription[$idOneList])){
					$addlists[-1][] = $idOneList;
				}
			}
		}

		//Now we have everything to be able to add the subscription
		if(!empty($addlists)) {
			$listsubClass = acymailing_get('class.listsub');
			$listsubClass->addSubscription($subid,$addlists);
		}
	}

	public function onAfterRoute(){

		if(empty($_POST['option']) OR empty($_POST['page']) OR $_POST['option'] != 'com_virtuemart' OR !in_array($_POST['page'],array('account.index','checkout.index'))) return;
		//We are on the checkout index page, so we can check the params and subscribe the user if it's required...

		//First we load the params if it's not done
		if(!isset($this->params)){
			$plugin =& JPluginHelper::getPlugin('system', 'vmacymailing');
			$this->params = new JParameter( $plugin->params );
		}

		$this->_subscribeUser($_REQUEST);
	}
}//endclass
